<?php

class Vimeoplayer extends Tools_Plugins_Abstract {

    const API_VIMEO = 'https://api.vimeo.com/';

    const PLAYER_VIMEO_URL = 'https://player.vimeo.com/';

    /**
     * per_page = 100 It is max value
     */
    const VIDEOS_PER_PAGE = '100';

    /**
     * viemo player default width
     */
    const VIDEO_DEFAULT_WIDTH = 600;

    /**
     * viemo player default height
     */
    const VIDEO_DEFAULT_HEIGHT = 400;

    const VIMEOPLAYER_SECURE_TOKEN = 'VimeoplayerToken';

    /**
     * Toaster layout
     *
     * @var
     */
    private $_layout = null;

    protected $_vimeoSettingsMapper = null;

    public function  __construct($options, $seotoasterData)
    {
        parent::__construct($options, $seotoasterData);
        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());
        $this->_vimeoSettingsMapper = Vimeoplayer_Models_Mappers_VimeoplayerSettingMapper::getInstance();

        if (($scriptPaths = Zend_Layout::getMvcInstance()->getView()->getScriptPaths()) !== false) {
            $this->_view->setScriptPath($scriptPaths);
        }
        $this->_view->addScriptPath(__DIR__ . '/system/views/');
    }

    public function configAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isPost()) {
                $configParams = filter_var_array($this->_request->getParams(), FILTER_SANITIZE_STRING);
                $accessToken = $this->_request->getParam('accessToken');

                $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
                $tokenValid = Tools_System_Tools::validateToken($secureToken, self::VIMEOPLAYER_SECURE_TOKEN);
                if (!$tokenValid) {
                    $this->_responseHelper->fail('');
                }
                $vimeoSettings = $this->_vimeoSettingsMapper->getConfigParams();

                if (empty($accessToken)) {
                    $configParams['accessToken'] = '';
                    if (isset($vimeoSettings['accessToken'])) {
                        $configParams['accessToken'] = $vimeoSettings['accessToken'];
                    }
                }

                if (empty($accessToken)) {
                    $this->_responseHelper->fail($this->_translator->translate('Vimeo Player not configured!'));
                }

                $this->_vimeoSettingsMapper->save($configParams);

                $this->_responseHelper->success('');
            } else {
                $vimeoSettings = $this->_vimeoSettingsMapper->getConfigParams();
                $this->_view->settings = $vimeoSettings;
                $this->_view->translator = $this->_translator;
                $this->_layout->content = $this->_view->render('vimeoConfig.phtml');
                echo $this->_layout->render();
            }
        }
    }

    public function _makeOptionPlayer()
    {
        if(!empty($this->_options[1])){
            $name = Tools_VimeoplayerTools::toHash($this->_options[0]. $this->_options[1] . __CLASS__);
            $pageId = (end($this->_options) == 'static') ? 0 : $this->_seotoasterData['id'];
            $type = ($pageId) ? Application_Model_Models_Container::TYPE_REGULARCONTENT : Application_Model_Models_Container::TYPE_STATICCONTENT;
            $container = Application_Model_Mappers_ContainerMapper::getInstance()->findByName($name, $pageId, $type);
            $vimeoSettings = $this->_vimeoSettingsMapper->getConfigParams();

            if(!empty($vimeoSettings['accessToken']) && !empty($vimeoSettings['userId'])){

                $vimeoplayerSearchWorkflowsDbtable = new Vimeoplayer_Models_Dbtables_VimeoplayerSearchWorkflowsDbtable();
                $vimeoplayerSearchWorkflows = $vimeoplayerSearchWorkflowsDbtable->fetchAll()->toArray();

                $fields = '&fields=uri,name,description,pictures.sizes.width,pictures.sizes.height,pictures.sizes.link';
                $paramsQuery = 'users/'. $vimeoSettings['userId']. '/videos?per_page='. self::VIDEOS_PER_PAGE.'&access_token='. $vimeoSettings['accessToken'];

                if(empty($vimeoplayerSearchWorkflows)){
                    $this->processingFullData($paramsQuery, $fields);
                }else{
                    $savedDate = strtotime("+120 hours", strtotime($vimeoplayerSearchWorkflows[0]['lastUpdatedTime']));
                    $currentDate = strtotime('now');

                    if($savedDate < $currentDate){
                        $vimeoplayerSearchWorkflowsDbtable->delete('');
                        $this->processingFullData($paramsQuery, $fields);
                    }
                }

                if ($container instanceof Application_Model_Models_Container) {
                    $data = Zend_Json::decode($container->getContent());

                    if(!empty($data['channelId'])){
                        $allVideos = array();

                        $fields = '&fields=uri,name,description,pictures.sizes.width,pictures.sizes.height,pictures.sizes.link';
                        $paramsQuery = 'channels/'. $data['channelId']. '/videos/?access_token='. $vimeoSettings['accessToken']. '&per_page='. self::VIDEOS_PER_PAGE. $fields;

                        $vimeoplayerWorkflowsMapper = Vimeoplayer_Models_Mappers_VimeoplayerWorkflowsMapper::getInstance();

                        $vimeoplayerWorkflowsModel = $vimeoplayerWorkflowsMapper->getByChannelId($data['channelId']);
                        if($vimeoplayerWorkflowsModel instanceof Vimeoplayer_Models_Models_VimeoplayerWorkflowsModel){
                            $savedDate = strtotime("+12 hours", strtotime($vimeoplayerWorkflowsModel->getLastUpdatedTime()));
                            $currentDate = strtotime('now');

                            if($savedDate < $currentDate){
                                $vimeoContent = Tools_VimeoplayerTools::apiGet($paramsQuery, self::API_VIMEO);

                                if(!empty($vimeoContent) && !empty($vimeoContent['data'])){
                                    $firstVideo = $vimeoContent['data'][0];
                                    $this->_view->firstVideo = $firstVideo;
                                    $allVideos[0] = $vimeoContent['data'];

                                    if(!empty($vimeoContent['paging']['next'])){
                                        $additionalVideos = $this->processAdditionalVideos($vimeoContent, $paramsQuery);

                                        if(!empty($additionalVideos)){
                                            $allVideos = array_merge($allVideos, $additionalVideos);
                                        }
                                    }

                                    $vimeoplayerWorkflowsModel->setContent(Zend_Json::encode($allVideos));
                                    $vimeoplayerWorkflowsModel->setLastUpdatedTime(date('Y-m-d H:i:s', strtotime('now')));

                                    $vimeoplayerWorkflowsMapper->save($vimeoplayerWorkflowsModel);
                                }
                            }else{
                                $vimeoContent =  Zend_Json::decode($vimeoplayerWorkflowsModel->getContent());

                                $firstVideo = $vimeoContent[0][0];
                                $this->_view->firstVideo = $firstVideo;
                                $allVideos = $vimeoContent;
                            }
                        }else{
                            $vimeoContent = Tools_VimeoplayerTools::apiGet($paramsQuery, self::API_VIMEO);

                            if(!empty($vimeoContent) && !empty($vimeoContent['data'])){
                                $firstVideo = $vimeoContent['data'][0];
                                $this->_view->firstVideo = $firstVideo;
                                $allVideos[0] = $vimeoContent['data'];

                                if(!empty($vimeoContent['paging']['next'])){
                                    $additionalVideos = $this->processAdditionalVideos($vimeoContent, $paramsQuery);

                                    if(!empty($additionalVideos)){
                                        $allVideos = array_merge($allVideos, $additionalVideos);
                                    }
                                }

                                $vimeoplayerWorkflowsModel = new Vimeoplayer_Models_Models_VimeoplayerWorkflowsModel();

                                $vimeoplayerWorkflowsModel->setChannelId($data['channelId']);
                                $vimeoplayerWorkflowsModel->setContent(Zend_Json::encode($allVideos));
                                $vimeoplayerWorkflowsModel->setLastUpdatedTime(date('Y-m-d H:i:s', strtotime('now')));

                                $vimeoplayerWorkflowsMapper->save($vimeoplayerWorkflowsModel);
                            }
                        }

                        $this->_view->vimeoVideos = $allVideos;
                        $this->_view->showTitle = (!empty($data['title'])) ? $data['title'] : 0;
                        $this->_view->autoplay = (!empty($data['autoplay'])) ? $data['autoplay'] : 0;
                        $this->_view->width = (!empty($data['width'])) ? $data['width'] : self::VIDEO_DEFAULT_WIDTH;
                        $this->_view->height = (!empty($data['height'])) ? $data['height'] : self::VIDEO_DEFAULT_HEIGHT;

                        $this->_view->channelId = $data['channelId'];

                        $this->_view->isAdmin = false;

                        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
                            $this->_view->isAdmin = true;
                        }
                    }
                }
                $this->_view->pluginStylesFlag = false;

                if(!empty($vimeoSettings['pluginStyles'])){
                    $this->_view->pluginStylesFlag = true;
                }

                $this->_view->containerName = $name;
                $this->_view->pageId = $pageId;

                return $this->_view->render('vimeo-player.phtml');
            }
        }
        return $this->_translator->translate('Vimeo Player not configured!');
    }

    /**
     * Save Full data fo search video
     * @param $paramsQuery
     * @param $fields
     */
    public function processingFullData($paramsQuery, $fields){
        $vimeoplayerSearchWorkflowsMapper = Vimeoplayer_Models_Mappers_VimeoplayerSearchWorkflowsMapper::getInstance();

        $vimeoFullContent = Tools_VimeoplayerTools::apiGet($paramsQuery.$fields, self::API_VIMEO);

        if(!empty($vimeoFullContent) && !empty($vimeoFullContent['data'])){
            $allFullVideos[0] = $vimeoFullContent['data'];

            if(!empty($vimeoFullContent['paging']['next'])){
                $additionalVideos = $this->processAdditionalVideos($vimeoFullContent, $paramsQuery);

                if(!empty($additionalVideos)){
                    $allFullVideos = array_merge($allFullVideos, $additionalVideos);
                }
            }

            foreach ($allFullVideos as $videos){
                foreach ($videos as $key => $video){
                    $preparedData = array(
                        'videoId' => str_replace('videos/', '', stristr($video['uri'], 'videos/')),
                        'videoName' => $video['name'],
                        'content'   => Zend_Json::encode($video),
                        'lastUpdatedTime' => date('Y-m-d H:i:s', strtotime('now'))
                    );

                    $vimeoplayerSearchWorkflowsMapper->save($preparedData);
                }
            }
        }
    }

    public function processAdditionalVideos($vimeoContent, $paramsQuery){
        $countIterations = $vimeoContent['total'] / $vimeoContent['per_page'];
        if($countIterations > 1){
            $countIterations = round($countIterations);

            $additionalVideosUrls = array();

            for ($i = 1; $i <= $countIterations; $i) {
                $i++;
                $additionalVidUrl = $paramsQuery.'&page='.$i;
                $additionalVideosUrls[$i] = $additionalVidUrl;
            }

            if(!empty($additionalVideosUrls)){
                foreach ($additionalVideosUrls as $key => $url){
                    $videosPerPage = Tools_VimeoplayerTools::apiGet($url, self::API_VIMEO);

                    if(!empty($videosPerPage['data'])){
                        $allVideos[$key] = $videosPerPage['data'];
                    }
                }
            }
        }

        if(!empty($allVideos)){
            return $allVideos;
        }
        return array();
    }

    public function getVideoAction(){
        if ($this->_request->isPost()) {
            $vimeoVideoId = filter_var($this->_request->getParam('vimeoVideoId'), FILTER_SANITIZE_NUMBER_INT);
            $vimeoChanelId = filter_var($this->_request->getParam('vimeoChanel'), FILTER_SANITIZE_STRING);
            $vimeoSearchFlag = $this->_request->getParam('vimeoSearchFlag');

            $vimeoSettings = $this->_vimeoSettingsMapper->getConfigParams();

            if (empty($vimeoSettings['accessToken'])) {
                $this->_responseHelper->fail($this->_translator->translate('Vimeo Player not configured!'));
            }

            if(empty($vimeoSearchFlag)){
                if(empty($vimeoVideoId) || empty($vimeoChanelId)){
                    $this->_responseHelper->fail($this->_translator->translate('Can\'t open video'));
                }
                $vimeoplayerWorkflowsMapper = Vimeoplayer_Models_Mappers_VimeoplayerWorkflowsMapper::getInstance();

                $vimeoplayerWorkflowsModel = $vimeoplayerWorkflowsMapper->getByChannelId($vimeoChanelId);

                if($vimeoplayerWorkflowsModel instanceof Vimeoplayer_Models_Models_VimeoplayerWorkflowsModel){
                    $vimeoContent =  Zend_Json::decode($vimeoplayerWorkflowsModel->getContent());
                    $videoName = '';

                    foreach ($vimeoContent as $contentArr){
                        foreach ($contentArr as $content){
                            $videoUriId = str_replace('videos/', '', stristr($content['uri'], 'videos/'));
                            if($vimeoVideoId == $videoUriId){
                                $videoName =  $content['name'];
                            }
                        }
                    }

                    if(!empty($videoName)){
                        $vimeoContent = array();
                        $vimeoContent['name'] = $videoName;
                    }

                }
            }else{
                if(empty($vimeoVideoId)){
                    $this->_responseHelper->fail($this->_translator->translate('Can\'t open video'));
                }

                $vimeoplayerSearchWorkflowsMapper = Vimeoplayer_Models_Mappers_VimeoplayerSearchWorkflowsMapper::getInstance();
                $vimeoContent = $vimeoplayerSearchWorkflowsMapper->findByVideoId($vimeoVideoId);
            }

            if(!empty($vimeoContent)){
                $this->_responseHelper->success(array('vimeoVideo' => $vimeoContent));
            }else{
                $this->_responseHelper->fail($this->_translator->translate('Can\'t open video'));
            }
        }
    }

    public function vimeoplayerAction() {
        $containerName = filter_var($this->_request->getParam('containerName'), FILTER_SANITIZE_STRING);
        $pageId        = filter_var($this->_request->getParam('pageId'), FILTER_SANITIZE_NUMBER_INT);
        $type          = ($pageId) ? Application_Model_Models_Container::TYPE_REGULARCONTENT : Application_Model_Models_Container::TYPE_STATICCONTENT;
        $container     = Application_Model_Mappers_ContainerMapper::getInstance()->findByName($containerName, $pageId, $type);
        $data        = array();

        if($container instanceof Application_Model_Models_Container) {
            $data = Zend_Json::decode($container->getContent());
        }

        if(is_array($data) && !empty($data)) {
            foreach($data as $key => $value) {
                $this->_view->$key = $value;
            }
        }

        $this->_view->pageId        = $pageId;
        $this->_view->containerName = $containerName;

        $this->_layout->content = $this->_view->render('vimeo.builder.phtml');
        echo $this->_layout->render();
    }

    public function _makeOptionSearch(){
        $vimeoSettings = $this->_vimeoSettingsMapper->getConfigParams();

        if(!empty($vimeoSettings['accessToken']) && !empty($vimeoSettings['userId'])){
            $this->_view->pluginStylesFlag = false;

            if(!empty($vimeoSettings['pluginStyles'])){
                $this->_view->pluginStylesFlag = true;
            }

            return $this->_view->render('vimeo-player-search.phtml');
        }
        return $this->_translator->translate('Vimeo Player not configured!');
    }

    public function searchVideoAction(){
        if ($this->_request->isPost()) {
            $vimeoSettings = $this->_vimeoSettingsMapper->getConfigParams();

            $searchParam = $this->_request->getParam('vimeoPlayerSearch');

            if(!empty($vimeoSettings['accessToken']) && !empty($vimeoSettings['userId']) && !empty($searchParam)){
                $vimeoplayerSearchWorkflowsMapper = Vimeoplayer_Models_Mappers_VimeoplayerSearchWorkflowsMapper::getInstance();
                $vimeoplayerSearchWorkflows = $vimeoplayerSearchWorkflowsMapper->findByVideoName($searchParam);

                if(!empty($vimeoplayerSearchWorkflows)){
                    $videoIds = array();
                    foreach ($vimeoplayerSearchWorkflows as $key => $video){
                        $videoIds['vimeo_player-'.$video['videoId']] = 'vimeo_player-'.$video['videoId'];
                        $vimeoplayerSearchWorkflows[$key]['content'] = Zend_Json::decode($video['content']);
                    }

                    $firstVideo = current($vimeoplayerSearchWorkflows);

                    $videosWithContainerId = $vimeoplayerSearchWorkflowsMapper->findVideosContainers($videoIds);

                    if(!empty($videosWithContainerId)){
                        foreach ($videosWithContainerId as $key => $containerId){
                            $key = str_replace('vimeo_player-', '', $key);
                            $vimeoplayerSearchWorkflows[$key]['containerid'] = $containerId;
                        }
                    }

                    $this->_view->firstVideo = $firstVideo;

                    $this->_view->vimeoVideos = $vimeoplayerSearchWorkflows;
                    $this->_view->showTitle = 1;
                    $this->_view->autoplay = 0;
                    $this->_view->width = self::VIDEO_DEFAULT_WIDTH;
                    $this->_view->height = self::VIDEO_DEFAULT_HEIGHT;

                    $this->_view->isAdmin = false;

                    if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
                        $this->_view->isAdmin = true;
                    }

                    $searchResult = $this->_view->render('vimeo-player-search-result.phtml');

                    $themeData = Zend_Registry::get('theme');
                    $extConfig = Zend_Registry::get('extConfig');
                    $parserOptions = array(
                        'websiteUrl' => $this->_websiteHelper->getUrl(),
                        'websitePath' => $this->_websiteHelper->getPath(),
                        'currentTheme' => $extConfig['currentTheme'],
                        'themePath' => $themeData['path'],
                    );
                    $parser = new Tools_Content_Parser($searchResult, array(), $parserOptions);

                    $searchResult = $parser->parseSimple();

                    $this->_responseHelper->success(array('searchResult' => $searchResult, 'vimeoVideos' => $vimeoplayerSearchWorkflows));

                }else{
                    $this->_responseHelper->fail($this->_translator->translate('Nothing found'));
                }
            }
        }
    }

}